import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _initializerDefineProperty from "@babel/runtime/helpers/initializerDefineProperty";
import _createClass from "@babel/runtime/helpers/createClass";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/initializerWarningHelper";
var _dec, _class, _descriptor;
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { createModuleEventBus, makeObservable, obx } from '@alilc/lowcode-editor-core';
import { BasicContext } from './base-context';
import { EditorWindow } from './editor-window/context';
import { Resource } from './resource';
import { ResourceType } from './resource-type';
var event;
(function (event) {
  event["ChangeWindow"] = "change_window";
  event["ChangeActiveWindow"] = "change_active_window";
})(event || (event = {}));
var CHANGE_EVENT = 'resource.list.change';
export var Workspace = (_dec = obx.ref, (_class = /*#__PURE__*/function () {
  function Workspace(registryInnerPlugin, shellModelFactory) {
    this.registryInnerPlugin = registryInnerPlugin;
    this.shellModelFactory = shellModelFactory;
    this.context = void 0;
    this.emitter = createModuleEventBus('workspace');
    this._isActive = false;
    this.windows = [];
    this.editorWindowMap = new Map();
    _initializerDefineProperty(this, "window", _descriptor, this);
    this.resourceTypeMap = new Map();
    this.resourceList = [];
    this.init();
    makeObservable(this);
  }
  var _proto = Workspace.prototype;
  _proto.init = function init() {
    this.initWindow();
    this.context = new BasicContext(this, '');
  };
  _proto.initWindow = function initWindow() {
    if (!this.defaultResourceType) {
      return;
    }
    var title = this.defaultResourceType.name;
    var resource = new Resource({}, this.defaultResourceType, this);
    this.window = new EditorWindow(resource, this, title);
    this.editorWindowMap.set(this.window.id, this.window);
    this.windows.push(this.window);
    this.emitChangeWindow();
    this.emitChangeActiveWindow();
  };
  _proto.setActive = function setActive(value) {
    this._isActive = value;
  };
  _proto.registerResourceType = /*#__PURE__*/function () {
    var _registerResourceType = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(resourceTypeModel) {
      var resourceType;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            if (resourceTypeModel.resourceType === 'editor') {
              resourceType = new ResourceType(resourceTypeModel);
              this.resourceTypeMap.set(resourceTypeModel.resourceName, resourceType);
              if (!this.window && this.defaultResourceType) {
                this.initWindow();
              }
            }
          case 1:
          case "end":
            return _context.stop();
        }
      }, _callee, this);
    }));
    function registerResourceType(_x) {
      return _registerResourceType.apply(this, arguments);
    }
    return registerResourceType;
  }();
  _proto.getResourceList = function getResourceList() {
    return this.resourceList;
  };
  _proto.setResourceList = function setResourceList(resourceList) {
    var _this = this;
    this.resourceList = resourceList.map(function (d) {
      return new Resource(d, _this.getResourceType(d.resourceName), _this);
    });
    this.emitter.emit(CHANGE_EVENT, resourceList);
  };
  _proto.onResourceListChange = function onResourceListChange(fn) {
    var _this2 = this;
    this.emitter.on(CHANGE_EVENT, fn);
    return function () {
      _this2.emitter.off(CHANGE_EVENT, fn);
    };
  };
  _proto.getResourceType = function getResourceType(resourceName) {
    return this.resourceTypeMap.get(resourceName);
  };
  _proto.removeResourceType = function removeResourceType(resourceName) {
    if (this.resourceTypeMap.has(resourceName)) {
      this.resourceTypeMap["delete"](resourceName);
    }
  };
  _proto.removeEditorWindowById = function removeEditorWindowById(id) {
    var index = this.windows.findIndex(function (d) {
      return d.id === id;
    });
    this.remove(index);
  };
  _proto.remove = function remove(index) {
    var window = this.windows[index];
    this.windows.splice(index, 1);
    if (this.window === window) {
      this.window = this.windows[index] || this.windows[index + 1] || this.windows[index - 1];
      this.emitChangeActiveWindow();
    }
    this.emitChangeWindow();
  };
  _proto.removeEditorWindow = function removeEditorWindow(resourceName, title) {
    var index = this.windows.findIndex(function (d) {
      return d.resource.name === resourceName && d.title;
    });
    this.remove(index);
  };
  _proto.openEditorWindowById = function openEditorWindowById(id) {
    var window = this.editorWindowMap.get(id);
    if (window) {
      this.window = window;
      this.emitChangeActiveWindow();
    }
  };
  _proto.openEditorWindow = function openEditorWindow(name, title, options, viewType) {
    var resourceType = this.resourceTypeMap.get(name);
    if (!resourceType) {
      console.error(name + " is not available");
      return;
    }
    var filterWindows = this.windows.filter(function (d) {
      return d.resource.name === name && d.title == title;
    });
    if (filterWindows && filterWindows.length) {
      this.window = filterWindows[0];
      this.emitChangeActiveWindow();
      return;
    }
    var resource = new Resource({}, resourceType, this);
    this.window = new EditorWindow(resource, this, title, options);
    this.windows.push(this.window);
    this.editorWindowMap.set(this.window.id, this.window);
    this.emitChangeWindow();
    this.emitChangeActiveWindow();
  };
  _proto.onChangeWindows = function onChangeWindows(fn) {
    var _this3 = this;
    this.emitter.on(event.ChangeWindow, fn);
    return function () {
      _this3.emitter.removeListener(event.ChangeWindow, fn);
    };
  };
  _proto.emitChangeWindow = function emitChangeWindow() {
    this.emitter.emit(event.ChangeWindow);
  };
  _proto.emitChangeActiveWindow = function emitChangeActiveWindow() {
    this.emitter.emit(event.ChangeActiveWindow);
  };
  _proto.onChangeActiveWindow = function onChangeActiveWindow(fn) {
    var _this4 = this;
    this.emitter.on(event.ChangeActiveWindow, fn);
    return function () {
      _this4.emitter.removeListener(event.ChangeActiveWindow, fn);
    };
  };
  _createClass(Workspace, [{
    key: "skeleton",
    get: function get() {
      return this.context.innerSkeleton;
    }
  }, {
    key: "plugins",
    get: function get() {
      return this.context.innerPlugins;
    }
  }, {
    key: "isActive",
    get: function get() {
      return this._isActive;
    }
  }, {
    key: "defaultResourceType",
    get: function get() {
      if (this.resourceTypeMap.size >= 1) {
        return Array.from(this.resourceTypeMap.values())[0];
      }
      return null;
    }
  }]);
  return Workspace;
}(), (_descriptor = _applyDecoratedDescriptor(_class.prototype, "window", [_dec], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class));